@@ -127,6 +127,8 @@ static void gpa_paper_preview_item_finalize (GObject *object);
 static void gpa_paper_preview_item_update (GnomeCanvasItem *item, gdouble *affine, ArtSVP *clip, gint flags);
 static void gpa_paper_preview_item_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
 
+static void gpa_paper_preview_item_disconnect (GpaPaperPreviewItem *pp);
+
 static void gppi_hline (GnomeCanvasBuf *buf, gint y, gint xs, gint xe, guint32 rgba);
 static void gppi_vline (GnomeCanvasBuf *buf, gint x, gint ys, gint ye, guint32 rgba);
 static void gppi_rect (GnomeCanvasBuf *buf, gint xs, gint ys, gint xe, gint ye, guint32 rgba);
@@ -192,7 +194,6 @@ static void
 gpa_paper_preview_item_finalize (GObject *object)
 {
 	GpaPaperPreviewItem *pp;
-	gint i, max;
 
 	pp = GPA_PAPER_PREVIEW_ITEM (object);
 
@@ -212,14 +213,13 @@ gpa_paper_preview_item_finalize (GObject *object)
 		pp->right = NULL;
 	}
 
-	max = sizeof (pp->node) / sizeof (GPANode *);
-	for (i = 0; i < max; i++) {
-		g_signal_handler_disconnect (pp->node[i],
-					     pp->handler[i]);
-		gpa_node_unref (pp->node[i]);
-		pp->handler[i] = 0;
-		pp->node[i] = NULL;
-	}
+	gpa_paper_preview_item_disconnect (pp);
+
+	g_signal_handler_disconnect (G_OBJECT (pp->gpa_config), pp->handler_config);
+	pp->handler_config = 0;
+	pp->gpa_config = NULL;
+
+	pp->config = gnome_print_config_unref (pp->config);
 
 	G_OBJECT_CLASS (item_parent_class)->finalize (object);
 }
@@ -796,6 +796,7 @@ gpa_paper_preview_item_disconnect (GpaPaperPreviewItem *pp)
 						     pp->handler[i]);
 			pp->handler[i] = 0;
 			gpa_node_unref (pp->node[i]);
+			pp->node[i] = NULL;
 		}
 	}
 }
@@ -848,7 +849,6 @@ gpa_paper_preview_item_new (GnomePrintConfig *config, GtkWidget *canvas)
 	pp->gpa_config = GNOME_PRINT_CONFIG_NODE (pp->config);
 	pp->handler_config = g_signal_connect (G_OBJECT (pp->gpa_config), "modified",(GCallback)
 					       gpa_paper_preview_item_config_modified_cb, pp);
-	
 	gpa_paper_preview_item_load_colors (pp, canvas);
 	g_signal_connect (G_OBJECT (canvas), "style_set", G_CALLBACK (gpa_paper_preview_item_style_color_cb), pp);
 	gpa_paper_preview_item_reload_values (NULL, 0, pp);
